defmodule Nulla.Models.MediaAttachment do
  use Ecto.Schema
  import Ecto.Changeset
  alias Nulla.Repo
  alias Nulla.Snowflake
  alias Nulla.Models.Note

  @primary_key {:id, :integer, autogenerate: false}
  schema "media_attachments" do
    field :type, :string
    field :mediaType, :string
    field :url, :string
    field :name, :string
    field :width, :integer
    field :height, :integer

    belongs_to :note, Note

    timestamps(type: :utc_datetime)
  end

  def changeset(media_attachment, attrs) do
    media_attachment
    |> cast(attrs, [:type, :mediaType, :url, :name, :width, :height, :note_id])
    |> validate_required([:type, :mediaType, :url, :note_id])
  end

  def create_media_attachment(attrs) when is_map(attrs) do
    id = Map.get(attrs, :id, Snowflake.next_id())

    %__MODULE__{}
    |> changeset(attrs)
    |> put_change(:id, id)
    |> Repo.insert()
  end
end
