defmodule Nulla.Utils do
  alias Nulla.Models.Actor
  alias Nulla.Models.InstanceSettings

  def resolve_local_actor("https://" <> _ = uri) do
    case URI.parse(uri).path do
      "/@" <> username ->
        instance_settings = InstanceSettings.get_instance_settings!()
        domain = instance_settings.domain

        case Actor.get_actor(username, domain) do
          nil -> {:error, :not_found}
          user -> user
        end

      _ ->
        {:error, :invalid_actor}
    end
  end

  def fetch_remote_actor(uri) do
    request =
      Finch.build(:get, uri, [
        {"Accept", "application/activity+json"}
      ])

    case Finch.request(request, Finch) do
      {:ok, %Finch.Response{status: 200, body: body}} ->
        case Jason.decode(body) do
          {:ok, data} -> {:ok, data}
          _ -> {:error, :invalid_json}
        end

      _ ->
        {:error, :actor_fetch_failed}
    end
  end
end
