defmodule NullaWeb.NoteController do
  use NullaWeb, :controller
  alias Nulla.Repo
  alias Nulla.ActivityPub
  alias Nulla.Models.Note

  def show(conn, %{"username" => username, "id" => id}) do
    case Integer.parse(id) do
      {int_id, ""} ->
        note = Note.get_note(int_id) |> Repo.preload([:actor, :media_attachments])

        cond do
          is_nil(note) ->
            conn
            |> put_status(:not_found)
            |> json(%{error: "Not Found"})
            |> halt()

          username != note.actor.preferredUsername ->
            conn
            |> put_status(:not_found)
            |> json(%{error: "Not Found"})
            |> halt()

          true ->
            accept = List.first(get_req_header(conn, "accept"))

            if accept in ["application/activity+json", "application/ld+json"] do
              conn
              |> put_resp_content_type("application/activity+json")
              |> json(ActivityPub.note(note))
            else
              render(conn, :show, note: note, layout: false)
            end
        end

      _ ->
        conn
        |> put_status(:not_found)
        |> json(%{error: "Not Found"})
        |> halt()
    end
  end
end
